import pytest
from unittest.mock import Mock


@pytest.fixture(scope="function")
def signed_message_mock() -> Mock:
    mock = Mock()
    mock.message_hash = "mocked_hash"
    mock.v = 27
    mock.r = 0x0000000000000000000000000000000000000000000000000000000000000000
    mock.s = 0x0000000000000000000000000000000000000000000000000000000000000001

    return mock


@pytest.fixture(scope="function")
def signed_tx_mock() -> Mock:
    mock = Mock()
    mock.hash = "mocked_hash"
    mock.v = 28
    mock.r = 0x0000000000000000000000000000000000000000000000000000000000000002
    mock.s = 0x0000000000000000000000000000000000000000000000000000000000000003

    return mock


@pytest.fixture(scope="function")
def account_mock(signed_message_mock: Mock, signed_tx_mock: Mock) -> Mock:
    mock = Mock()
    mock.sign_message.return_value = signed_message_mock
    mock.sign_transaction.return_value = signed_tx_mock

    return mock



import pytest

from eth_hub.aws.boto3_wrappers.exceptions import CantDeleteKeyAwsError
from eth_hub.aws.exceptions import CantRemoveKey
from eth_hub.aws.key_store import AwsKeyStore
from .aws_mock import AwsMock, Key


def test_remove_key(
        key_manager: AwsKeyStore,
        mocked_key: Key,
        aws_mock_with_key: AwsMock,
) -> None:
    # when
    key_manager.remove_key(mocked_key.id)

    # then
    aws_mock_with_key.schedule_key_deletion_mock.assert_called_once()


def test_remove_key_with_exception(
        key_manager: AwsKeyStore,
        mocked_key: Key,
        aws_mock_with_key: AwsMock,
) -> None:
    # given
    aws_mock_with_key.schedule_key_deletion_mock.side_effect = CantDeleteKeyAwsError

    # when / then
    with pytest.raises(CantRemoveKey):
        key_manager.remove_key(mocked_key.id)



from eth_hub.local.key_storage import LocalKeyStorage


def test_create_key() -> None:
    # given
    local_signer = LocalKeyStorage()

    # when
    key_info = local_signer.create_key()

    # then
    assert key_info.id in local_signer._accounts



from typing import Optional

from botocore.exceptions import ClientError


class BaseAwsError(Exception):
    message = "Base error"

    def __init__(self, client_error: Optional[ClientError] = None) -> None:
        if client_error is None:
            self.code = ""

        else:
            response = client_error.response
            self.code = response.get("Error", {}).get("Code", "")

        super().__init__(f"{self.message}: {self.code}")


class CantCreateKeyObjectAwsError(BaseAwsError):
    message = "Can't create key"


class CantImportKeyMaterialAwsError(BaseAwsError):
    message = "Can't import key"


class CantGetAddressAwsError(BaseAwsError):
    message = "Can't get address"


class CantDeleteKeyAwsError(BaseAwsError):
    message = "Can't delete key"


class CantListKeysAwsError(BaseAwsError):
    message = "Can't list keys"


class CantListAliasesAwsError(BaseAwsError):
    message = "Can't list aliases"


class CantGetKeyInfoAwsError(BaseAwsError):
    message = "Can't get key info"


class CantSignMessage(BaseAwsError):
    message = "Can't sign message"



from uuid import UUID, uuid4

from eth_account import Account
from eth_account.signers.local import LocalAccount
from eth_typing import Hash32
from typing_extensions import override

from eth_hub.base_key_storage import BaseKeyStore
from eth_hub.local.key import LocalKey
from eth_hub.signatureinfo import SignatureInfo


class LocalKeyStorage(BaseKeyStore):
    def __init__(self) -> None:
        self._accounts: dict[UUID, LocalAccount] = {}

    @override
    def import_key(self, private_key: bytes) -> LocalKey:
        account: LocalAccount = Account.from_key(private_key)
        return self._add_account(account)

    @override
    def create_key(self) -> LocalKey:
        account: LocalAccount = Account.create()
        return self._add_account(account)

    @override
    def get_key(self, key_id: UUID) -> LocalKey:
        account: LocalAccount = self._accounts[key_id]
        address = self._get_bytes_address(account.address)
        return LocalKey(id=key_id, address=address)

    @override
    def list_keys(self) -> list[LocalKey]:
        return [
            LocalKey(id=key_id, address=self._get_bytes_address(account.address))
            for key_id, account in self._accounts.items()
        ]

    @override
    def remove_key(self, key_id: UUID) -> None:
        self._accounts.pop(key_id)

    @override
    def sign_hash(self, key_id: UUID, hash_: bytes) -> SignatureInfo:
        account: LocalAccount = self._accounts[key_id]
        hash32: Hash32 = Hash32(hash_)
        signed_message = account.unsafe_sign_hash(hash32)
        return SignatureInfo(
            key_id=key_id,
            hash=signed_message.message_hash,
            v=signed_message.v,
            r=signed_message.r,
            s=signed_message.s,
        )

    def _add_account(self, account: LocalAccount) -> LocalKey:
        key_id: UUID = uuid4()
        self._accounts[key_id] = account
        address = bytes.fromhex(account.address.removeprefix("0x"))

        return LocalKey(id=key_id, address=address)

    def _get_bytes_address(self, address: str) -> bytes:
        return bytes.fromhex(address.removeprefix("0x"))



from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.serialization import load_der_public_key
from eth_keys.datatypes import PublicKey


def public_key_to_address(public_key: bytes, safe: bool = True) -> bytes:
    if safe:
        der_public_key = load_der_public_key(public_key, backend=default_backend())
        uncompressed_public_key = der_public_key.public_bytes(
            encoding=serialization.Encoding.X962,
            format=serialization.PublicFormat.UncompressedPoint,
        )[1:]

    else:
        uncompressed_public_key = public_key[-64:]

    return PublicKey(uncompressed_public_key).to_canonical_address()



from uuid import uuid4

from asn1crypto.algos import DSASignature

from eth_hub.aws.key_store import AwsKeyStore

from .aws_mock import AwsMock, Key


def test_sign_hash(
    key_manager: AwsKeyStore,
    mocked_key: Key,
    aws_mock: AwsMock,
) -> None:
    # given
    address = "0x707cd01a8fc30c503d10ccfb8fe0c6670817b814"
    r = 77499307603103543196277082859912786141927594951515498733161022132679134249953
    s = 52246846724921623538504070280221495554738823431252421718508664272925772355973
    v = 1

    key_id = uuid4()
    aws_mock.keys[key_id] = Key(id=key_id, address=address)
    aws_mock.dsa_signatures.append(DSASignature({"r": r, "s": s}))

    hash_ = bytes.fromhex(
        "fdca2948a3f973fdfd6e0574a1980793d8fa2f157ab9866937029c7efc69aca2"
    )

    # when
    signature = key_manager.sign_hash(key_id, hash_)

    # then
    assert signature.r == r
    assert signature.s == s
    assert signature.v == v



from uuid import uuid4

import boto3
import pytest
from eth_account import Account
from mypy_boto3_kms import KMSClient
from pytest_mock import MockerFixture

from eth_hub.aws.key_store import AwsKeyStore

from .aws_mock import AwsMock, Key


@pytest.fixture
def client() -> KMSClient:
    return boto3.client("kms", region_name="eu-west-2")


@pytest.fixture
def key_manager(client: KMSClient) -> AwsKeyStore:
    return AwsKeyStore(client)


@pytest.fixture
def aws_mock(mocker: MockerFixture) -> AwsMock:
    return AwsMock(mocker)


@pytest.fixture
def mocked_key() -> Key:
    return Key(
        id=uuid4(),
        address=Account.create().address,
    )


@pytest.fixture
def aws_mock_with_key(mocked_key: Key, aws_mock: AwsMock, client: KMSClient) -> AwsMock:
    aws_mock.keys[mocked_key.id] = mocked_key
    return aws_mock



from typing import Optional
from uuid import UUID

import boto3
from eth_keys import keys
from mypy_boto3_kms import KMSClient
from pydantic import SecretBytes
from typing_extensions import override

from eth_hub.aws.boto3_wrappers.dto import KeyState
from eth_hub.aws.boto3_wrappers.exceptions import BaseAwsError
from eth_hub.aws.boto3_wrappers.key import (
    create_key_item,
    fulfil_private_key,
    get_address,
    get_key_ids,
    schedule_key_deletion,
)
from eth_hub.aws.boto3_wrappers.metadata import (
    check_alias_already_taken,
    get_aliases,
    get_key_metadata,
    set_alias,
)
from eth_hub.aws.boto3_wrappers.signature import sign_message
from eth_hub.aws.exceptions import (
    AliasAlreadyTakenError,
    CantCreateKeyError,
    CantFindValidVError,
    CantGetKeyInfoError,
    CantListKeysError,
    CantRemoveKey,
    CantSetAlias,
    CantSignHash,
    KeyNotFound,
)
from eth_hub.aws.key import AwsKey
from eth_hub.base_key_storage import BaseKeyStore
from eth_hub.signatureinfo import SignatureInfo

SECP256K1_ORDER = 0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEBAAEDCE6AF48A03BBFD25E8CD0364141


class AwsKeyStore(BaseKeyStore):
    def __init__(self, boto3_client: Optional[KMSClient] = None):
        self.boto3_client = boto3_client or boto3.client("kms")

    @override
    def import_key(self, private_key: bytes) -> AwsKey:
        try:
            key_id = create_key_item(
                client=self.boto3_client,
                create_key_by_aws=False,
            )
            fulfil_private_key(
                client=self.boto3_client,
                key_id=key_id,
                private_key=SecretBytes(private_key),
            )
        except BaseAwsError as error:
            raise CantCreateKeyError(error)

        return self.get_key(key_id=key_id)

    @override
    def create_key(self) -> AwsKey:
        try:
            key_id = create_key_item(
                client=self.boto3_client,
                create_key_by_aws=True,
            )
        except BaseAwsError as error:
            raise CantCreateKeyError(error)

        return self.get_key(key_id=key_id)

    @override
    def get_key(self, key_id: UUID) -> AwsKey:
        try:
            if key := self._get_key(key_id, only_enabled=False):
                return key
        except BaseAwsError as error:
            raise CantGetKeyInfoError(error)

        raise KeyNotFound

    @override
    def list_keys(self, only_enabled: bool = True) -> list[AwsKey]:
        try:
            return [
                key
                for key_id in get_key_ids(client=self.boto3_client)
                if (key := self._get_key(key_id, only_enabled))
            ]

        except BaseAwsError as error:
            raise CantListKeysError(error)

    @override
    def remove_key(self, key_id: UUID) -> None:
        try:
            schedule_key_deletion(client=self.boto3_client, key_id=key_id)
        except BaseAwsError as error:
            raise CantRemoveKey(error)

    @override
    def sign_hash(self, key_id: UUID, hash_: bytes) -> SignatureInfo:
        try:
            dsa_signature = sign_message(
                client=self.boto3_client,
                key_id=key_id,
                message_hash=hash_,
            )
        except BaseAwsError as error:
            raise CantSignHash(error)

        r: int = dsa_signature["r"].native  # type: ignore
        s: int = dsa_signature["s"].native  # type: ignore

        if s > SECP256K1_ORDER // 2:
            s = SECP256K1_ORDER - s

        v = self._find_v(key_id, hash_, r, s)

        return SignatureInfo(key_id=key_id, hash=hash_, v=v, r=r, s=s)

    def set_alias(self, key_id: UUID, alias: str) -> None:
        try:
            if check_alias_already_taken(self.boto3_client, alias):
                raise AliasAlreadyTakenError

            set_alias(client=self.boto3_client, key_id=key_id, alias=alias)
        except BaseAwsError as error:
            raise CantSetAlias(error)

    def _find_v(self, key_id: UUID, message_hash: bytes, r: int, s: int) -> int:
        address = get_address(client=self.boto3_client, key_id=key_id)

        for v in (0, 1):
            signature = keys.Signature(vrs=(v, r, s))
            recovered_address = signature.recover_public_key_from_msg_hash(message_hash)

            if recovered_address.to_canonical_address() == address:
                return v

        raise CantFindValidVError

    def _get_key(self, key_id: UUID, only_enabled: bool) -> Optional[AwsKey]:
        metadata = get_key_metadata(client=self.boto3_client, key_id=key_id)

        if only_enabled and metadata.key_state != KeyState.ENABLED:
            return None

        aliases = get_aliases(client=self.boto3_client, key_id=key_id)
        address = get_address(client=self.boto3_client, key_id=key_id)
        return AwsKey(id=metadata.key_id, address=address, aliases=aliases or [])



import datetime
from uuid import UUID

from botocore.exceptions import ClientError
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import ec, padding
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicKey
from mypy_boto3_kms import KMSClient
from mypy_boto3_kms.literals import (
    AlgorithmSpecType,
    DataKeyPairSpecType,
    WrappingKeySpecType,
)
from pydantic import SecretBytes

from eth_hub.aws.boto3_wrappers.dto import (
    CreateKeyItemResponse,
    GetPublicKeyResponse,
    ListKeysPage,
    ScheduleDeletionResponse,
)
from eth_hub.aws.boto3_wrappers.exceptions import (
    CantCreateKeyObjectAwsError,
    CantDeleteKeyAwsError,
    CantGetAddressAwsError,
    CantImportKeyMaterialAwsError,
    CantListKeysAwsError,
)
from eth_hub.aws.boto3_wrappers.utils import public_key_to_address

CUSTOMER_MASTER_KEY_SPEC: DataKeyPairSpecType = "ECC_SECG_P256K1"
WRAPPING_ALGORITHM: AlgorithmSpecType = "RSAES_OAEP_SHA_256"
WRAPPING_KEY_SPEC: WrappingKeySpecType = "RSA_2048"


def create_key_item(client: KMSClient, create_key_by_aws: bool) -> UUID:
    try:
        response_raw = client.create_key(
            KeyUsage="SIGN_VERIFY",
            CustomerMasterKeySpec=CUSTOMER_MASTER_KEY_SPEC,
            Origin="AWS_KMS" if create_key_by_aws else "EXTERNAL",
        )
    except ClientError as error:
        raise CantCreateKeyObjectAwsError(error)

    response = CreateKeyItemResponse.model_validate(response_raw)
    return response.key_metadata.key_id


def fulfil_private_key(
    client: KMSClient, key_id: UUID, private_key: SecretBytes
) -> None:
    import_params = client.get_parameters_for_import(
        KeyId=str(key_id),
        WrappingAlgorithm=WRAPPING_ALGORITHM,
        WrappingKeySpec=WRAPPING_KEY_SPEC,
    )

    wrapping_public_key = import_params["PublicKey"]
    import_token = import_params["ImportToken"]

    encrypted_key_material = _wrap_private_key(wrapping_public_key, private_key)

    try:
        client.import_key_material(
            KeyId=str(key_id),
            ImportToken=import_token,
            EncryptedKeyMaterial=encrypted_key_material,
            ExpirationModel="KEY_MATERIAL_DOES_NOT_EXPIRE",
        )
    except ClientError as error:
        raise CantImportKeyMaterialAwsError(error)


def get_key_ids(client: KMSClient) -> list[UUID]:
    try:
        paginator = client.get_paginator("list_keys")
        return [
            key.key_id
            for page in paginator.paginate()
            for key in ListKeysPage.model_validate(page).keys
        ]
    except ClientError as error:
        raise CantListKeysAwsError(error)


def get_address(client: KMSClient, key_id: UUID) -> bytes:
    try:
        response_raw = client.get_public_key(KeyId=str(key_id))
    except ClientError as error:
        raise CantGetAddressAwsError(error)

    response = GetPublicKeyResponse.model_validate(response_raw)
    return public_key_to_address(response.public_key)


def schedule_key_deletion(
    client: KMSClient, key_id: UUID, days_period=7
) -> datetime.datetime:
    try:
        response = client.schedule_key_deletion(
            KeyId=str(key_id), PendingWindowInDays=days_period
        )
    except ClientError as error:
        raise CantDeleteKeyAwsError(error)

    return ScheduleDeletionResponse.model_validate(response).deletion_datetime


def _wrap_private_key(wrapping_public_key: bytes, private_key: SecretBytes) -> bytes:
    backend = default_backend()
    int_private_key = int.from_bytes(private_key.get_secret_value(), byteorder="big")

    der_key = ec.derive_private_key(int_private_key, ec.SECP256K1(), backend)

    der_private_key = der_key.private_bytes(
        encoding=serialization.Encoding.DER,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption(),
    )

    wrapping_public_key_der = serialization.load_der_public_key(
        wrapping_public_key, backend
    )

    if not isinstance(wrapping_public_key_der, RSAPublicKey):
        raise TypeError("Wrapping key must be an RSA public key.")

    return wrapping_public_key_der.encrypt(
        der_private_key,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None,
        ),
    )



import pytest

from eth_hub.aws.boto3_wrappers.exceptions import CantCreateKeyObjectAwsError, CantListAliasesAwsError, \
    CantGetAddressAwsError, CantListKeysAwsError
from eth_hub.aws.exceptions import CantGetKeyInfoError, CantListKeysError
from eth_hub.aws.key_store import AwsKeyStore
from .aws_mock import AwsMock, Key


def test_get_key(
        mocked_key: Key,
        key_manager: AwsKeyStore,
        aws_mock_with_key: AwsMock,
) -> None:
    # when
    key_manager.get_key(mocked_key.id)

    # then
    aws_mock_with_key.get_key_metadata_mock.assert_called_once()
    aws_mock_with_key.get_aliases_mock.assert_called_once()
    aws_mock_with_key.get_address_mock.assert_called_once()
    aws_mock_with_key.get_key_ids_mock.assert_not_called()


def test_get_key_with_exception_on_getting_metadata(
        mocked_key: Key,
        key_manager: AwsKeyStore,
        aws_mock_with_key: AwsMock,
) -> None:
    # given
    aws_mock_with_key.get_key_metadata_mock.side_effect = CantCreateKeyObjectAwsError

    # when / then
    with pytest.raises(CantGetKeyInfoError):
        key_manager.get_key(mocked_key.id)


def test_get_key_with_exception_on_getting_aliases(
        mocked_key: Key,
        key_manager: AwsKeyStore,
        aws_mock_with_key: AwsMock,
) -> None:
    # given
    aws_mock_with_key.get_aliases_mock.side_effect = CantListAliasesAwsError

    # when / then
    with pytest.raises(CantGetKeyInfoError):
        key_manager.get_key(mocked_key.id)


def test_get_key_with_exception_on_getting_addresses(
        mocked_key: Key,
        key_manager: AwsKeyStore,
        aws_mock_with_key: AwsMock,
) -> None:
    # given
    aws_mock_with_key.get_address_mock.side_effect = CantGetAddressAwsError

    # when / then
    with pytest.raises(CantGetKeyInfoError):
        key_manager.get_key(mocked_key.id)


def test_list_keys(
        key_manager: AwsKeyStore,
        aws_mock_with_key: AwsMock,
) -> None:
    # when
    keys = key_manager.list_keys()

    # then
    assert len(keys) == 1
    aws_mock_with_key.get_key_metadata_mock.assert_called_once()
    aws_mock_with_key.get_aliases_mock.assert_called_once()
    aws_mock_with_key.get_address_mock.assert_called_once()
    aws_mock_with_key.get_key_ids_mock.assert_called_once()


def test_list_keys_with_exception_on_getting_ids(
        key_manager: AwsKeyStore,
        aws_mock_with_key: AwsMock,
) -> None:
    # given
    aws_mock_with_key.get_key_ids_mock.side_effect = CantListKeysAwsError

    # when / then
    with pytest.raises(CantListKeysError):
        key_manager.list_keys()



import uuid

from botocore.stub import Stubber
from mypy_boto3_kms import KMSClient
from eth_hash.auto import keccak

from eth_hub.aws.boto3_wrappers.signature import sign_message

def test_set_alias(client: KMSClient, stubber: Stubber) -> None:
    # given
    key_id = uuid.uuid4()
    signature = bytes.fromhex(
        "3046022100eb6c5e5503afdb89e0580c3aa25069323d8a050a707be5b08d115c084bd9cb"
        "1a022100a64ba0a49b3ac89eb67ef01fc1c1348bb46ae0b9caa06964b723ac426535f818"
    )
    msg_hash = keccak(b"test value")

    stubber.add_response(
        method="sign",
        service_response={
            "KeyId": f"arn:aws:kms:eu-west-2:111111111111:key/{key_id}",
            "Signature": signature,
            "SigningAlgorithm": "ECDSA_SHA_256",
            "ResponseMetadata": {
                "HTTPStatusCode": 200
            }
        }
    )

    # when
    signature_received = sign_message(client, key_id, msg_hash)

    # then
    stubber.assert_no_pending_responses()
    assert signature_received["r"].native == 0xeb6c5e5503afdb89e0580c3aa25069323d8a050a707be5b08d115c084bd9cb1a
    assert signature_received["s"].native == 0xa64ba0a49b3ac89eb67ef01fc1c1348bb46ae0b9caa06964b723ac426535f818



from typing import Generator

import pytest
from botocore.stub import Stubber
from mypy_boto3_kms import KMSClient


@pytest.fixture
def stubber(client: KMSClient) -> Generator[Stubber, None, None]:
    stubber = Stubber(client)
    stubber.activate()

    yield stubber

    stubber.deactivate()



import datetime
from typing import Optional
from uuid import UUID, uuid4

from asn1crypto.algos import DSASignature
from eth_account import Account
from mypy_boto3_kms.client import KMSClient
from pydantic import BaseModel, ConfigDict, SecretBytes
from pytest_mock import MockerFixture

from eth_hub.aws.boto3_wrappers.dto import KeyMetadata, KeySpec, KeyState
from eth_hub.aws.boto3_wrappers.exceptions import (
    CantCreateKeyObjectAwsError,
    CantGetAddressAwsError,
)


class Key(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    id: UUID
    aliases: list[str] = []
    address: Optional[str] = None
    key_state: KeyState = KeyState.ENABLED
    removal_scheduled: bool = False
    enabled: bool = True


class AwsMock:
    def __init__(self, mocker: MockerFixture) -> None:
        self.keys: dict[UUID, Key] = {}
        self.dsa_signatures: list[DSASignature] = []

        self.create_key_item_mock = mocker.patch(
            "eth_hub.aws.key_store.create_key_item", side_effect=self.create_key_item
        )
        self.fulfil_private_key_mock = mocker.patch(
            "eth_hub.aws.key_store.fulfil_private_key",
            side_effect=self.fulfil_private_key,
        )
        self.get_key_ids_mock = mocker.patch(
            "eth_hub.aws.key_store.get_key_ids", side_effect=self.get_key_ids
        )
        self.get_address_mock = mocker.patch(
            "eth_hub.aws.key_store.get_address", side_effect=self.get_address
        )
        self.schedule_key_deletion_mock = mocker.patch(
            "eth_hub.aws.key_store.schedule_key_deletion",
            side_effect=self.schedule_key_deletion,
        )
        self.set_alias_mock = mocker.patch(
            "eth_hub.aws.key_store.set_alias", side_effect=self.set_alias
        )
        self.get_aliases_mock = mocker.patch(
            "eth_hub.aws.key_store.get_aliases", side_effect=self.get_aliases
        )
        self.check_alias_already_taken_mock = mocker.patch(
            "eth_hub.aws.key_store.check_alias_already_taken",
            side_effect=self.check_alias_already_taken,
        )
        self.get_key_metadata_mock = mocker.patch(
            "eth_hub.aws.key_store.get_key_metadata", side_effect=self.get_key_metadata
        )
        self.sign_message_mock = mocker.patch(
            "eth_hub.aws.key_store.sign_message", side_effect=self.sign_message
        )

    def create_key_item(self, client: KMSClient, create_key_by_aws: bool) -> UUID:
        key_id = uuid4()
        self.keys[key_id] = Key(id=key_id)
        if create_key_by_aws:
            self.keys[key_id].address = Account.create().address

        return key_id

    def fulfil_private_key(
        self, client: KMSClient, key_id: UUID, private_key: SecretBytes
    ) -> None:
        self.keys[key_id].address = Account.from_key(
            private_key.get_secret_value()
        ).address

    def get_key_ids(self, client: KMSClient) -> list[UUID]:
        return list(self.keys.keys())

    def get_address(self, client: KMSClient, key_id: UUID) -> bytes:
        if (key := self.keys[key_id]) and key.address is not None:
            return bytes.fromhex(key.address.removeprefix("0x"))

        raise CantGetAddressAwsError()

    def schedule_key_deletion(
        self, client: KMSClient, key_id: UUID, days_period=7
    ) -> datetime.datetime:
        self.keys[key_id].removal_scheduled = True
        return datetime.datetime.now(datetime.UTC) + datetime.timedelta(
            days=days_period
        )

    def set_alias(self, client: KMSClient, key_id: UUID, alias: str) -> None:
        if self.check_alias_already_taken(client, alias):
            raise CantCreateKeyObjectAwsError()

        self.keys[key_id].aliases.append(alias)

    def get_aliases(self, client: KMSClient, key_id: UUID) -> list[str]:
        return self.keys[key_id].aliases

    def check_alias_already_taken(self, client: KMSClient, alias: str) -> bool:
        return any(alias in key.aliases for key in list(self.keys.values()))

    def get_key_metadata(self, client: KMSClient, key_id: UUID) -> KeyMetadata:
        return KeyMetadata(
            KeyId=key_id,
            Enabled=self.keys[key_id].enabled,
            KeyState=self.keys[key_id].key_state,
            Description="",
            CustomerMasterKeySpec=KeySpec.ECC_SECG_P256K1,
        )

    def sign_message(
        self, client: KMSClient, key_id: UUID, message_hash: bytes
    ) -> DSASignature:
        return self.dsa_signatures.pop()



from uuid import UUID

from asn1crypto.algos import DSASignature  # type: ignore
from botocore.exceptions import ClientError
from mypy_boto3_kms import KMSClient

from eth_hub.aws.boto3_wrappers.dto import SignResponse
from eth_hub.aws.boto3_wrappers.exceptions import CantSignMessage


def sign_message(client: KMSClient, key_id: UUID, message_hash: bytes) -> DSASignature:
    try:
        response = client.sign(
            KeyId=str(key_id),
            Message=message_hash,
            MessageType="DIGEST",
            SigningAlgorithm="ECDSA_SHA_256",
        )
    except ClientError as error:
        raise CantSignMessage(error)

    der_signature = SignResponse.model_validate(response).signature
    return DSASignature.load(der_signature)



from abc import ABC, abstractmethod
from typing import Sequence
from uuid import UUID

from eth_hub.base_key import BaseKey
from eth_hub.signatureinfo import SignatureInfo


class BaseKeyStore(ABC):
    @abstractmethod
    def import_key(self, private_key: bytes) -> BaseKey:
        pass

    @abstractmethod
    def create_key(self) -> BaseKey:
        pass

    @abstractmethod
    def get_key(self, key_id: UUID) -> BaseKey:
        pass

    @abstractmethod
    def list_keys(self) -> Sequence[BaseKey]:
        pass

    @abstractmethod
    def remove_key(self, key_id: UUID) -> None:
        pass

    @abstractmethod
    def sign_hash(self, key_id: UUID, hash_: bytes) -> SignatureInfo:
        pass
